"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var isPred_1 = require("./isPred");
var isClause_1 = require("./isClause");
var isClauseRef_1 = require("./isClauseRef");
var isDelayedClause_1 = require("./isDelayedClause");
var Clause_1 = require("../models/Clause");
var Problem_1 = require("../models/Problem");
var fnName_1 = require("./fnName");
var stringifyWithFnName_1 = require("./stringifyWithFnName");
var CLAUSE_TYPE_PRED = "PRED";
function coerceIntoClause(expr) {
    if (isClause_1.default(expr) || isClauseRef_1.default(expr) || isDelayedClause_1.default(expr)) {
        return expr;
    }
    else if (isPred_1.default(expr)) {
        return _wrap(expr);
    }
    else {
        console.error(expr);
        throw new Error("'Expression must either be a Clause object or a predication function that returns true or false. Given value: " + stringifyWithFnName_1.default(expr) + "'");
    }
}
exports.default = coerceIntoClause;
function _wrap(pred) {
    return new Clause_1.default({
        type: CLAUSE_TYPE_PRED,
        exprs: [pred],
        opts: {
            predicate: pred,
        },
        conformFn: predConformer(pred),
        generateFn: null,
    });
}
function predConformer(pred) {
    return function conformPred(x) {
        if (pred(x)) {
            return x;
        }
        else {
            return new Problem_1.default(x, pred, [], "Predicate " + fnName_1.default(pred) + " returns false");
        }
    };
}
//# sourceMappingURL=coerceIntoClause.js.map